package edu.uoc.android.uocapp;

import edu.uoc.android.uocapp.R;
import edu.uoc.android.uocapp.util.BrowserUOC;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Toast;

/* 
 *  Clase de la aplicacion encargada de solicitar el usuario y la contrasea.
 *  Se loga en el portal de la UOC y establece las variables de sesin y cookies para ser empleadas mas adelante.
 */
public class ActCredenciales extends Activity {

	private ProgressDialog mPD;
	private EditText EditLogin;
	private EditText EditPass;
	private CheckBox cbRecordar;
	private boolean enProceso=false;

	// Para evitar que nuestra actividad comience de nuevo al rotar la pantalla
	// Ver en el manifest, ya que adems es necesario que la Activity tenga:
	// android:configChanges="keyboard|keyboardHidden|orientation">        
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
	super.onConfigurationChanged(newConfig);
	}
		
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);		
		setContentView(R.layout.lcredenciales);
		Log.i("UOC","Comienzo UOCApp...");

		// Asignamos los objetos a los elementos
		EditLogin =(EditText) findViewById(R.id.editLogin);
		EditPass  =(EditText) findViewById(R.id.editPass);
		cbRecordar = (CheckBox) findViewById(R.id.checkRemember);
		// Obtenemos los valores del perfil privado de la aplicacin...
		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
		EditLogin.setText (settings.getString("login", ""));
		boolean  bRecordar = settings.getBoolean("recordar", false);
		cbRecordar.setChecked(bRecordar);

		if (bRecordar) { // Si est recordada...
			EditPass.setText (settings.getString("pass", ""));
		}        

		Button btnAcceso = (Button)this.findViewById(R.id.btnLogin);

		btnAcceso.setOnClickListener(new OnClickListener(){ 
			public void onClick(View arg0) { 

				// Si estamos ya en proceso o no tenemos conexin a Internet Salimos
				if (enProceso || !checkInternet())
					return;
				
				// Evitaremos as varias llamadas a la funcin...
				enProceso=true;
				if (EditLogin.getText().toString().equals("") || EditPass.getText().toString().equals(""))
				{
					mostrarMensaje(getString(R.string.noPasa));
					enProceso=false;
				}
				else
				{
					Log.i("UOC","Sacado Progress...");
					mPD=ProgressDialog.show(ActCredenciales.this, "", getText(R.string.accediendo));
					// Guardamos las preferencias         		
					SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
					SharedPreferences.Editor editor = settings.edit();
					editor.putString("login",EditLogin.getText().toString());
					boolean bRecordar = cbRecordar.isChecked();
					editor.putBoolean("recordar",bRecordar);

					if (bRecordar){ // y la contrasea si se ha pedido...
						editor.putString("pass",EditPass.getText().toString());
					}
					editor.commit();
					
					// Arrancamos un hilo para logarnos, cerraremos el ProgressDialog al final del proceso.
					Thread hilo = new Thread(new Runnable(){
						public void run(){
							Log.i("UOC","Thread:Run: Mensaje enviado...");
							handler.sendEmptyMessage(0);						
						}										
					});
					hilo.start();
				}
			}});
	}

	// Definimos un manejador ya que dentro del hilo no se puede operar con la UI (da errores)
	// http://www.helloandroid.com/tutorials/using-threads-and-progressdialog
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			Log.i("UOC","handleMessage; llamamos a LoginUOC...");		
			LoginUOC(EditLogin.getText().toString().toLowerCase(),EditPass.getText().toString());				
		}
	};

	// Mtodo para comprobar si tenemos conexin a Internet. No es estrictamente necesario
	// ya que en caso de no tener la clase conexin muestra el mensaje correspondiente, pero esto ahorra tiempo
	// al usuario en caso de no tener conexin ya que muestra el error antes.
	private boolean checkInternet()
	{
		ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo comprueboInternet = cm.getActiveNetworkInfo();
		if (comprueboInternet != null && comprueboInternet.isConnectedOrConnecting()) {
			return true;
		}

		else {
			mostrarMensaje(getString(R.string.ErrorIO));
			return false;
		}
	}
	
	// Mtodo para Logar el usuario en el portal de la UOC
	private void LoginUOC (String sLogin, String sPass){

		boolean bErr=false;
		String sSession="";
		BrowserUOC mBrowser = new BrowserUOC(ActCredenciales.this);
		
		mBrowser.setUrl("https://cv.uoc.edu/cgi-bin/uoc");
		mBrowser.addParam("l", sLogin);
		mBrowser.addParam("p", sPass);
		mBrowser.addParam("appid", "WUOC");
		mBrowser.addParam("nil", "XXXXXX");
		mBrowser.addParam("lb", "a");
		mBrowser.addParam("url", "https://cv.uoc.edu");
		mBrowser.addParam("x", "13");
		mBrowser.addParam("y", "2");

		Log.i("UOC","LoginUOC: Vamos a logarnos...");
		// Realizamos la peticin POST a la UOC con el usuario y el password
		if (mBrowser.doPost()) // Si no se producen errores investigaremos la pgina resultante...
		{
			String resp=mBrowser.getPage();
			int iErr=resp.indexOf("var error = '");
			if (iErr>0)
			{   // Obtenemos el error del portal...
				int iFin=resp.indexOf("';",iErr);
				// Como el mensaje de usuario/contrasea incorrecto sale en catalan y con el <BR>, lo cambiamos...
				if (resp.substring(iErr+13, iFin).startsWith("El nom dusuari o la contrasenya sn incorrectes"))
				{
					mostrarMensaje(getString(R.string.ErrorLogin));
				}
				else
				{ // Mostramos el mensaje...
					mostrarMensaje(resp.substring(iErr+13, iFin));
				}
				bErr=true;
			}
			else
			{
				// Acceso correcto, obtenemos la cookie y el session  			    		
				Log.i("UOC","LoginUOC: Vamos a sacar el almacen de cookies...");
				if (mBrowser.getAlmacenCookies().isEmpty())
				{
					Log.i("UOC","LoginUOC: No hay cookies...");					
					mostrarMensaje(getString(R.string.ErrorRespuesta));
					bErr=true; // Para no continuar...
				} 
				Log.i("UOC","LoginUOC: Obtenido almacn de cookies...");

				// Obtenemos la sesin...
				int iSs=resp.indexOf("?s=");
				if (iSs>0)
				{   // Obtenemos el error del portal...
					int iSf=resp.indexOf("\";",iSs);
					sSession = resp.substring(iSs+3, iSf);
					mBrowser.setSessionUOC(sSession);
					Log.i("UOC","LoginUOC: Session:"+mBrowser.getSessionUOC());
				}
				else
				{
					// No podemos obtener la sesin... tal vez exista algn error....
					bErr=true; // Para no continuar...
					mostrarMensaje(getString(R.string.ErrorRespuesta));        			    		
				}
			}								
		}
		else
		{
			bErr=true;
		}

		if (!bErr)
		{ // No se han producido errores y tenemos todo lo necesario, continuamos...			
			Intent intent = new Intent(); // Haremos un Forward de actividad, esto significa que esta actividad desaparece.
			intent.setClass(ActCredenciales.this, ListaOpciones.class);
			mBrowser.cerrar();
			startActivity(intent);
			Log.i("UOC","LoginUOC: Fin Actividad 'ActCredenciales'.");
			finish();
		}
		// Cerramos el dialogo de proceso...
		mPD.dismiss();
		enProceso=false;
	}

	// Muestra un mensaje...
	private void mostrarMensaje(String texto)
	{
		Toast.makeText(ActCredenciales.this, texto, Toast.LENGTH_LONG).show();
	}
	
	
	// Mtodo llamado cuando se selecciona una opcin en el men
	public boolean onOptionsItemSelected (MenuItem item){
		boolean cambiaTiempo=false;
		int actualizarCada=0;
				
		switch (item.getItemId()){

		case R.id.nunca:
			actualizarCada=-1;
			cambiaTiempo=true;
			break;

		case R.id.diez:
			actualizarCada=60*10;
			cambiaTiempo=true;
			break;

		case R.id.minuto:
			actualizarCada=60;
			cambiaTiempo=true;
			break;

		case R.id.cincominutos:
			actualizarCada=60*5;
			cambiaTiempo=true;
			break;

		case R.id.mnMenu3:
			
		      LayoutInflater factory = LayoutInflater.from(this);
	            final View textEntryView = factory.inflate(R.layout.menu_addextra, null);
	             AlertDialog.Builder aD = new AlertDialog.Builder(ActCredenciales.this);
	                aD.setTitle(R.string.addExtra);
	                aD.setView(textEntryView);
            		EditText extraURL =(EditText) textEntryView.findViewById(R.id.extraurledit);
            		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
            		extraURL.setText (settings.getString("contenidosExtra", ""));
	                aD.setPositiveButton(R.string.alert_dialog_ok, new DialogInterface.OnClickListener() {
	                	// El usuario pulsa "Ok."
	                    public void onClick(DialogInterface dialog, int whichButton) {
	                		EditText extraURL =(EditText) textEntryView.findViewById(R.id.extraurledit);
	                		// Desde aqu no se puede acceder a las SharedPreferences, por eso hacemos un mtodo
	                    	cambiarOpcionesEx(extraURL.getText().toString());
	                    }
	                });
	                aD.setNegativeButton(R.string.alert_dialog_cancel, new DialogInterface.OnClickListener() {
	                    public void onClick(DialogInterface dialog, int whichButton) {
	                    	// El usuario pulsa "Cancelar"
	    					Log.i("UOC", "ActCredenciales:addItem - No guardamos contenidosExtra");
	    					dialog.dismiss();
	                    }
	                });
	             
	                AlertDialog alertD = aD.create();
	             	alertD.show();	             	
			break;
			
		case R.id.mnMenu4:
			finish();	
			Log.i("UOC","cerrar: Byebye.");
			break;
		} 	 

		if (cambiaTiempo) // El cambio de URL no se puede controlar aqui porque el AlertDialog no bloquea la ejecucin.
		{
		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();
		Log.i("UOC","ActCredenciales:onOptionsItemSelected: Grabamos preferencia ActualizarCada:"+actualizarCada);
		editor.putInt("updateEvery",actualizarCada);
		editor.commit();
		}
		Log.i("UOC","ActCredenciales:onOptionsItemSelected: Salimos del Menu.");
		return true;
	}

	// Mtodo invocado desde el AlertDialog para actualizar el valor de la URL de contenidos Extra.
	private void cambiarOpcionesEx(String url)
	{
		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();
	    editor.putString("contenidosExtra",url);			
	    Log.i("UOC", "ListaOpciones:addItem - Guardamos contenidosExtra: '"+url+"'");			
		editor.commit();		
	}
	// Mtodo para cargar el men a partir del XML menu.
	public boolean onCreateOptionsMenu(Menu menu){

		// Creamos el menu a partir del XML que tenemos en los recursos (menu\menu.xml)
		MenuInflater inflater=getMenuInflater();
		inflater.inflate(R.menu.menuppal,menu);
		return true;
	}
	
	
}

