package edu.uoc.android.uocapp;

import edu.uoc.android.uocapp.beans.Contenido;
import edu.uoc.android.uocapp.util.BrowserUOC;
import edu.uoc.android.uocapp.util.Parser;

import java.lang.Thread.State;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.Toast;

import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.Window;

/*
 * Clase que presenta la lista de aulas del alumno
 * El buzn personal y el calendario semanal
 */
public class ListaOpciones extends ListActivity implements Runnable {

	private boolean actualizando = false;
	private int actualizarCada = -1;
	private int tiempo;
	private Activity activity; 
	private boolean endAPP=false;
	private BrowserUOC mBrowser;
	private NotificationManager mNotificationManager=null;
	private Thread hilo;
	private final int NEWMSG_ID = 1; // Notificacin de nuevos mensajes y/o correos
	
	// Para evitar que nuestra actividad comience de nuevo al rotar la pantalla
	// Ver en el manifest, ya que adems es necesario que la Activity tenga:
	// android:configChanges="keyboard|keyboardHidden|orientation">        
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
	super.onConfigurationChanged(newConfig);
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {

		super.onCreate(savedInstanceState);		

		// Progress bar en la barra de la aplicacin...
		//requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
		activity = this; 
		activity.requestWindowFeature(Window.FEATURE_PROGRESS); 

		Log.i("UOC", "onCreate: Comienzo ListaOpciones...");
		actualizar();

		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
		actualizarCada = settings.getInt("updateEvery", -1);
		// Posibles valores: =<0: Nunca
		// 30, 60, ... Se actualiza cada "N" segundos.
		String valor="";
		if (actualizarCada <0)
			valor=(String)getText(R.string.ListaNoActualizar);
		else if (actualizarCada<60)
			valor=(String)getText(R.string.ListaActualizarCada)+ actualizarCada + " "+(String)getText(R.string.Segundos) +(String)getText(R.string.cambiarOpciones);
		else if (actualizarCada==60)
			valor=(String)getText(R.string.ListaActualizarCadaMinuto)+(String)getText(R.string.cambiarOpciones);		
		else if (actualizarCada>60)
			valor=(String)getText(R.string.ListaActualizarCada)+" " + (int)(actualizarCada/60)+" "+(String)getText(R.string.Minutos) + (String)getText(R.string.cambiarOpciones);	

		mostrarMensaje(valor);
		// Para actualizar cada "N" segundos...
		hilo = new Thread(this);
		hilo.start();
	}

	// Mtodo para refrescar la lista de elementos (Buzn, citas, aulas,...).
	private void actualizar(){

		Log.i("UOC","Actualizar: Actualizamos lista... (tiempo:"+this.tiempo+")");
		if (actualizando) return; // Ya estamos en ello, nos salimos.
		actualizando=true;
		//this.getWindow().setFeatureInt(Window.FEATURE_INDETERMINATE_PROGRESS, -1);
		Log.i("UOC","Actualizar: Comenzamos...");
		mBrowser = new BrowserUOC(ListaOpciones.this);
		Log.i("UOC", "Actualizar: Sesion Almacenada: '"+mBrowser.getSessionUOC()+"");		
		mBrowser.setUrl("https://cv.uoc.edu/UOC2000/b/cgi-bin/hola?s="+mBrowser.getSessionUOC()+"&newStartingPage=0");
		mBrowser.doGet();

		// Contenedor para los ITEMs de la Lista...
		List<Map<String, Object>> groupData = new ArrayList<Map<String, Object>>();

		// Objeto encargado de realizar el parseo de la pgina resultante.
		Parser mParser = new Parser(this, mBrowser.getSessionUOC());

		// Establecemos si hay contenidos Extra de donde descargarlos.
		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);		
		mParser.setUrlExtra(settings.getString("contenidosExtra", ""));
		
		// Si la pgina retornara un XML modificaramos esta clase nicamente, por eso se separa en una clase el parseo.
		ArrayList<Contenido> cnt = mParser.ObtenerContenidos(mBrowser.getPage());
		
		if (cnt!=null)
		{
		Iterator<Contenido> it = cnt.iterator();
		while (it.hasNext())
			{   // Item de la lista...
			Contenido tmp = it.next();
			String titulo=tmp.getsTitulo();			
			String contenido=tmp.getsSubtitulo();
			String url=tmp.getsURL();
			Log.i("UOC","Actualizar: Titulo: " + titulo + " - URL Aula: " + url);
			addItem(groupData, titulo, contenido, url);			
			}

		// Creamos el adaptador...
		SimpleAdapter adapter = new SimpleAdapter( this, groupData, android.R.layout.simple_list_item_2, 
				new String[] { "title", "cont"},
				new int[]{ android.R.id.text1, android.R.id.text2 } );
		setListAdapter( adapter );
		// Hemos terminado...
		}
		else
		{	// Si no se reciben contenidos es porque la sesin ha caducado...
			mostrarMensaje((String)getText(R.string.NoSession));
			Intent intent = new Intent();
			intent.setClass(ListaOpciones.this, ActCredenciales.class);
			mBrowser.cerrar();
			startActivity(intent);
			Log.i("UOC","Actualizar: Fin Actividad 'ListaOpciones'.");
			finish();			
		}
		mBrowser.cerrar();
		actualizando=false;
		
		// http://developer.android.com/guide/topics/ui/notifiers/notifications.html
		if (mParser.hayNuevosMensajes() || mParser.hayNuevosCorreos()) // Ponemos un aviso de nuevos mensajes...
		{
			Log.i("UOC", "Actualizar: Aviso de nuevos mensajes");
			String ns = Context.NOTIFICATION_SERVICE;
			mNotificationManager = (NotificationManager) getSystemService(ns);
			int icon = R.drawable.logo_uoc;
			CharSequence tickerText = null;
			Log.i("UOC", "Actualizar: Hay correos? " +mParser.hayNuevosCorreos());
			Log.i("UOC", "Actualizar: Hay mensajes? " +mParser.hayNuevosMensajes());
			
			if (mParser.hayNuevosMensajes() && mParser.hayNuevosCorreos())
			{
				tickerText=(String)getText(R.string.NuevosCorreosyMensajes);
			}
			else if (mParser.hayNuevosCorreos())				
			{
				tickerText=(String)getText(R.string.NuevosCorreos);				
			}
			else
			{
				tickerText=(String)getText(R.string.NuevosMensajes);				
			}
														
			long when = System.currentTimeMillis();
			Notification notification = new Notification(icon, tickerText, when);
			// Cuando nos llegue la notificacin sonar el sonido para ellas...
			notification.defaults |= Notification.DEFAULT_SOUND;

			Context context = getApplicationContext();			
			CharSequence contentTitle = (String)getText(R.string.NuevosMensajes);
			CharSequence contentText;
			if (mParser.hayNuevosMensajes() && mParser.hayNuevosCorreos())
			{
				contentText =(String)getText(R.string.NuevosCorreosyMensajes2);
			}
			else if (mParser.hayNuevosCorreos())				
			{
				contentText =(String)getText(R.string.NuevosCorreos2);				
			}
			else
			{
				contentText =(String)getText(R.string.NuevosMensajes2);				
			}

			Intent notificationIntent = new Intent(this, ListaOpciones.class);
			notificationIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);
			PendingIntent contentIntent = PendingIntent.getActivity(this, 0, notificationIntent, 0);
			notification.setLatestEventInfo(context, contentTitle, contentText, contentIntent);			
			mNotificationManager.notify(NEWMSG_ID, notification);
		}
		
	}	

	@Override
	public void onDestroy() {
	    Log.i("UOC", "onDestroy(): Borramos notificaciones de la barra de tareas.");
	    // Borramos las notificaciones...

	    try{
	    mNotificationManager.cancel(NEWMSG_ID);
	    }catch(Exception e)
	    {
	    	// No existan notificaciones, no hacemos nada... 
	    }
		// Cerrar la aplicacion...
		endAPP=true;
		Log.i("UOC","onDestroy: Esperando a terminar el hilo...");
		while (hilo.getState()!=State.TERMINATED); // Esperar...
		Log.i("UOC","onDestroy: Finalizamos Activity.");

	    super.onDestroy();
	}

	@Override
	public void onRestart() {
	    Log.i("UOC", "onRestart()");
	    super.onDestroy();
	}
	
	@Override
	public void onResume() {
	    Log.i("UOC", "onResume()");
	    super.onDestroy();
	}

	@Override
	public void onPause() {
	    Log.i("UOC", "onPause()");
	    super.onDestroy();
	}
	
	// Mtodo para aadir un item a la lista
	private void addItem(List<Map<String, Object>>  data, String titulo, String contenido, String url) {
		Map<String, Object> temp = new HashMap<String, Object>();
		temp.put("title", titulo);
		temp.put("cont", contenido);
		temp.put("url", url);
		data.add(temp);
	}

	// Mtodo invocado al seleccionar una opcin de la lista
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		Map map = (Map) l.getItemAtPosition(position);
		Log.i("UOC","Posicion:"+position);

		Intent myWebViewIntent = new Intent(ListaOpciones.this, UOCWebView.class);
		mBrowser.setUrl((String) map.get("url"));
		startActivity(myWebViewIntent);		
	}

	// Mtodo para mostrar un mensaje en pantalla.
	private void mostrarMensaje(String texto)
	{
		Log.i("UOC","mostrarMensaje: "+texto);		
		Toast.makeText(ListaOpciones.this, texto, Toast.LENGTH_LONG).show();
	}

	// Mtodo llamado cuando se selecciona una opcin en el men
	public boolean onOptionsItemSelected (MenuItem item){
		boolean cambiaTiempo=false;
				
		switch (item.getItemId()){
		case R.id.mnMenu1:   // Actualizar ahora....
			actualizar();	
			tiempo=0;
			break;

		case R.id.nunca:
			actualizarCada=-1;
			cambiaTiempo=true;
			tiempo=0;
			break;

		case R.id.diez:
			actualizarCada=60*10;
			cambiaTiempo=true;
			break;

		case R.id.minuto:
			actualizarCada=60;
			cambiaTiempo=true;
			break;

		case R.id.cincominutos:
			actualizarCada=60*5;
			cambiaTiempo=true;
			break;

		case R.id.mnMenu3:
			
		      LayoutInflater factory = LayoutInflater.from(this);
	            final View textEntryView = factory.inflate(R.layout.menu_addextra, null);
	             AlertDialog.Builder aD = new AlertDialog.Builder(ListaOpciones.this);
	                aD.setTitle(R.string.addExtra);
	                aD.setView(textEntryView);
            		EditText extraURL =(EditText) textEntryView.findViewById(R.id.extraurledit);
            		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
            		extraURL.setText (settings.getString("contenidosExtra", ""));
	                aD.setPositiveButton(R.string.alert_dialog_ok, new DialogInterface.OnClickListener() {
	                	// El usuario pulsa "Ok."
	                    public void onClick(DialogInterface dialog, int whichButton) {
	                		EditText extraURL =(EditText) textEntryView.findViewById(R.id.extraurledit);
	                		// Desde aqu no se puede acceder a las SharedPreferences, por eso hacemos un mtodo
	                    	cambiarOpcionesEx(extraURL.getText().toString());
	                    }
	                });
	                aD.setNegativeButton(R.string.alert_dialog_cancel, new DialogInterface.OnClickListener() {
	                    public void onClick(DialogInterface dialog, int whichButton) {
	                    	// El usuario pulsa "Cancelar"
	    					Log.i("UOC", "ListaOpciones:addItem - No guardamos contenidosExtra");
	    					dialog.dismiss();
	                    }
	                });
	             
	                AlertDialog alertD = aD.create();
	             	alertD.show();	             	
			break;
			
		case R.id.mnMenu4:
			cerrar();
			break;
		} 	 

		if (cambiaTiempo) // El cambio de URL no se puede controlar aqui porque el AlertDialog no bloquea la ejecucin.
		{
		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();
		Log.i("UOC","onOptionsItemSelected: Grabamos preferencia ActualizarCada:"+actualizarCada);
		editor.putInt("updateEvery",actualizarCada);
		editor.commit();
		}
		Log.i("UOC","onOptionsItemSelected: Salimos del Menu.");
		return true;
	}

	// Mtodo invocado desde el AlertDialog para actualizar el valor de la URL de contenidos Extra.
	private void cambiarOpcionesEx(String url)
	{
		SharedPreferences settings = getSharedPreferences("perfilUOC", MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();
	    editor.putString("contenidosExtra",url);			
	    Log.i("UOC", "ListaOpciones:addItem - Guardamos contenidosExtra: '"+url+"'");			
		editor.commit();		
	}
	// Mtodo para cargar el men a partir del XML menu.
	public boolean onCreateOptionsMenu(Menu menu){

		// Creamos el menu a partir del XML que tenemos en los recursos (menu\menu.xml)
		MenuInflater inflater=getMenuInflater();
		inflater.inflate(R.menu.menu,menu);
		return true;
	}

	public void run() {
		do
		{
			try {
				Thread.sleep(1000);
				handler.sendEmptyMessage(0);

			} catch (InterruptedException e) {
				Log.i("UOC","run: Hilo parado.");
			}
		}while (!endAPP);
		Log.i("UOC","run: El hilo sale.");
	}

	private void cerrar() {

		Log.i("UOC","cerrar: Cerrar sesin...");		
		mBrowser = new BrowserUOC(ListaOpciones.this);		
		mBrowser.setUrl("https://cv.uoc.edu/cgi-bin/byebye?s="+mBrowser.getSessionUOC());
		mBrowser.doGet();

		// Cerrar la aplicacion...
		endAPP=true;
		finish();	
		Log.i("UOC","cerrar: Byebye.");
		
	}

	// Definimos un manejador ya que dentro del hilo no se puede operar con la UI (da errores)
	// http://www.helloandroid.com/tutorials/using-threads-and-progressdialog

	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {

			//Log.i("UOC","handleMessage: Hilo...cada "+actualizarCada+" llevo "+tiempo);
			if (actualizarCada>0)
			{
				tiempo ++;
				//Log.i("UOC","handleMessage: Mayor q 0");
				if (tiempo>actualizarCada)
				{
					tiempo=0;
					//Log.i("UOC","handleMessage: Actualiza!");
					actualizar();
				}
				else
				{			
					//Log.i("UOC","handleMessage: Aun no...");
					int valor= 10000-(int)(tiempo*(10000/actualizarCada));
					//Log.i("UOC","handleMessage: Actualizar en:"+valor);
					getWindow().setFeatureInt(Window.FEATURE_PROGRESS, valor);
				}
			}
			else
			{	// Quitamos el progressBar...
				getWindow().setFeatureInt(Window.FEATURE_PROGRESS, 10000);
			}	 
		}
	};
}