package edu.uoc.android.uocapp;

import java.util.List;

import org.apache.http.cookie.Cookie;

import edu.uoc.android.uocapp.R;
import edu.uoc.android.uocapp.util.BrowserUOC;
import android.app.Activity;
import android.app.ProgressDialog;
import android.os.Bundle;
import android.util.Log;
import android.view.Window;
import android.webkit.CookieManager;
import android.webkit.CookieSyncManager;
import android.webkit.WebChromeClient;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.content.res.Configuration;

/*
 * Clase para realizar la navegacin web por el portal. 
 * Empleando las cookies obtenidas veremos las aulas, mensajes, calendario, etc.
 */
public class UOCWebView extends Activity {

	private BrowserUOC mBrowser;
	private ProgressDialog mPD;
	
	// http://developer.android.com/reference/android/webkit/WebView.html

	
	
	// Para evitar que nuestra actividad comience de nuevo al rotar la pantalla
	// Ver en el manifest, ya que adems es necesario que la Activity tenga:
	// android:configChanges="keyboard|keyboardHidden|orientation">        
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
	super.onConfigurationChanged(newConfig);
	}
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		// Para mostrar la el progreso de la carga de la pgina...
		getWindow().requestFeature(Window.FEATURE_PROGRESS);
        getWindow().setFeatureInt(Window.FEATURE_PROGRESS, Window.PROGRESS_VISIBILITY_ON);
		setContentView(R.layout.uoc_webview);
		
        Log.i("UOC", "onCreate: Comienzo de UOCWebView...");
		 
		// Lo instanciamos para obtener los objetos estticos (URL, Cookies).
		mBrowser = new BrowserUOC(UOCWebView.this);
		WebView wb = (WebView)findViewById(R.id.mainwebview);
		
		wb.getSettings().setBuiltInZoomControls(true); 
		wb.getSettings().setJavaScriptEnabled(true); 
		wb.getSettings().setJavaScriptCanOpenWindowsAutomatically(true);
		wb.getSettings().setSupportMultipleWindows(false);

		final Activity activity = this;
		wb.setWebChromeClient(new WebChromeClient() {
			@Override	        
			   public void onProgressChanged(WebView view, int progress) {
			     // Ponemos el progreso. La barra desaparecer al llegar al 100%
			     activity.setProgress(progress * 1000);
			   }
			 });

		
		wb.setWebViewClient(new WebViewClient(){
			
			@Override
        public boolean shouldOverrideUrlLoading(WebView view, String url) {
            Log.i("UOC", "shouldOverrideUrlLoading: Procesando URL...");
            view.loadUrl(url);
            return true;
        }
			@Override
        public void onPageFinished(WebView view, String url) {
            Log.i("UOC", "onPageFinished: " +url);
            if (mPD.isShowing()) {
            	mPD.dismiss();
            }
        }
        
			@Override
        public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
            Log.e("UOC", "onReceivedError: " + description);
        }
    });
    	
		mPD=ProgressDialog.show(UOCWebView.this, "", getText(R.string.accediendo2));
		
		// Sacamos las cookies...
		List<Cookie> cookies = mBrowser.getAlmacenCookies();

		if (! cookies.isEmpty()){

			CookieSyncManager.createInstance(null);
			CookieManager cookieManager = CookieManager.getInstance();

			// Sincronizamos las cookies del cliente HTTP
			for (Cookie cookie : cookies){

				Cookie sessionInfo = cookie;

				String cookieString = sessionInfo.getName() + "=" + sessionInfo.getValue() + "; domain=" + sessionInfo.getDomain();
				cookieManager.setCookie("cv.uoc.edu", cookieString);
				CookieSyncManager.getInstance().sync();
			}
		}	    
		setProgressBarVisibility(true);
		wb.loadUrl(mBrowser.getUrl());	
	}
}

