package edu.uoc.uocservicios.persistence;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import com.google.appengine.api.datastore.DatastoreService;
import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.FetchOptions;
import com.google.appengine.api.datastore.Query;
import com.google.appengine.api.datastore.FetchOptions.Builder;
import com.google.appengine.api.datastore.Query.SortDirection;


public class ServiciosUtils {


	// Restringimos el mximo nmero de resultados.
	private static final int FETCH_MAX_RESULTS = 100;  

	/** 
	 * Almacenamiento de un registro nuevo 
	 * @param Contenido cnt Contenido 
	 */  
	public static void insert(Contenido cnt) {  
		// Desde GAE no es posible insertar objetos, por ello lo realizamos aqu...  
		// recuperacion del datastore
		final DatastoreService datastoreService = DSF.getDatastoreService();		
		// lo insertamos en el datastore
		datastoreService.put(cnt.getEntity());

	}  

	/** 
	 * Borrado de un registro nuevo 
	 * @param int idcnt idContenido 
	 */  
	public static boolean delete(String idCnt) {  
		// Esta Operacin se realizar desde GAE
		return false;
	}  


	/** 
	 * Cambiar un contenido 
	 * @param int idcnt idContenido 
	 */  
	public static boolean modify(String idCnt, Contenido cnt) {  
		// Esta Operacin se realizar desde GAE
		return false;
	}  

	/** 
	 * Obtener de un Contenido
	 * @param idContenido id del contenido
	 */  
	public static Contenido obtener(String idContenido) {            
		// Esta Operacin se realizar desde GAE
		return null;
	}  

	/**
	 * Configuracion de la consulta a lanzar contra el datastore
	 * @param tipo {1: slo activos, 2:en fecha, resto:todos}
	 * @return una consulta configurada
	 */
	private static Query configureQuery (int tipo) {

		final Query query = new Query(Contenido.CONTENIDO_ENTITY);
		if (tipo==1)
		{
			query.addFilter(Contenido.FORZAR_ACTIVO, Query.FilterOperator.EQUAL, Boolean.TRUE);
		}
		else if (tipo==2)
		{		
			String fechaParam=""; 
			SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy");
			Date date = new Date();

			try {
				fechaParam = (formatter.format(date));
			}
			catch (Exception e) {}

			query.addFilter(Contenido.ACTIVO_DESDE, Query.FilterOperator.GREATER_THAN_OR_EQUAL, fechaParam);
			query.addFilter(Contenido.ACTIVO_HASTA, Query.FilterOperator.LESS_THAN_OR_EQUAL, fechaParam);
			query.addFilter(Contenido.FORZAR_ACTIVO, Query.FilterOperator.EQUAL, Boolean.FALSE);
		}
		query.addSort(Contenido.TITULO, SortDirection.DESCENDING);		
		return query;
	}

	/**
	 * Configuracion de las opciones de recuperacion de datos
	 * @return unas opciones de recuperacion de datos configuradas
	 */
	private static FetchOptions configureFetchOptions () {
		return Builder.withLimit(FETCH_MAX_RESULTS);	
	}

	/**
	 * Conversion de entidad a tutorial
	 * @param entity entidad a ser convertida en tutorial
	 * @return tutorial procedente de una entidad
	 */
	private static Contenido convertEntityToContenido (final Entity entity) {
		Contenido cnt = new Contenido();
		cnt.setId((String) entity.getProperty(Contenido.CID));		
		cnt.setTitulo((String) entity.getProperty(Contenido.TITULO));
		cnt.setSubtitulo((String) entity.getProperty(Contenido.SUBTITULO));
		cnt.setURL((String) entity.getProperty(Contenido.URL));
		cnt.setActivoDesde((Date) entity.getProperty(Contenido.ACTIVO_DESDE));
		cnt.setActivoHasta((Date) entity.getProperty(Contenido.ACTIVO_HASTA));
		cnt.setForzarActivo((Boolean) entity.getProperty(Contenido.FORZAR_ACTIVO));
		return cnt;
	}

	/** 
	 * Recuperacin de los contenidos activos
	 * @return una lista con los contenidos activos
	 */  
	public static List<Contenido> getAllEntries() {
		// recuperacion del datastore y configuracion de la consulta
		return getEntries(false);
	}

	public static List<Contenido> getOnlyActiveEntries() {
		// recuperacion del datastore y configuracion de la consulta
		return getEntries(true);
	}

	private static List<Contenido> getEntries(boolean soloVigentes) {
		// recuperacion del datastore y configuracion de la consulta
		final DatastoreService datastoreService = DSF.getDatastoreService();
		Query query =null;

		if (soloVigentes) {
			query = configureQuery(1);
		}
		else
		{
			query = configureQuery(3);		
		}
		FetchOptions fetchOptions = configureFetchOptions();

		// declaracion de un listado donde volcar los resultados
		final List<Contenido> contenidos = new ArrayList<Contenido>();

		// lanzamiento de la consulta, la cual recupera entidades
		for (Entity entity: datastoreService.prepare(query).asList(fetchOptions)) {
			// conversion de las entidades a tutoriales
			contenidos.add(convertEntityToContenido (entity));	
		}		
		/* Feature no soportada por Google:
	GRAVE: The RuntimeException could not be mapped to a response, re-throwing to the HTTP container
	java.lang.IllegalArgumentException: Only one inequality filter per query is supported.  
	Encountered both _activoDesde and _activoHasta
	at com.google.appengine.api.datastore.DatastoreApiHelper.translateError(DatastoreApiHelper.java:36)
	at com.google.appengine.api.datastore.DatastoreApiHelper$1.convertException(DatastoreApiHelper.java:98)
		
		
		if (soloVigentes) { // Aadios los viegentes por fecha...
			query = configureQuery(2);
			fetchOptions = configureFetchOptions();

			// lanzamiento de la consulta, la cual recupera entidades
			for (Entity entity: datastoreService.prepare(query).asList(fetchOptions)) {
				// conversion de las entidades a tutoriales
				contenidos.add(convertEntityToContenido (entity));	
			}		

		}
		*/


		return contenidos;
	}

}  